import uuid
from typing import Optional, Union

import click

from gcs_cli.helpers import record_print_result_message, requires_login
from gcs_cli.parsing import EXPLICIT_NULL, UUIDorDEFAULTorNull, globus_command
from gcs_cli.parsing.types import _ExplicitNullClass
from gcs_cli.services.gcs import get_client


@globus_command("set-subscription-id", short_help="Set an endpoint subscription ID")
@click.argument("SUBSCRIPTION_ID", type=UUIDorDEFAULTorNull())
@requires_login
def endpoint_set_subscription_id(
    subscription_id: Union[_ExplicitNullClass, str, uuid.UUID]
) -> None:
    """
    Update the subscription ID on the endpoint to which you are currently logged in

    Unlike setting the subscription ID via the 'endpoint update' command, this operation
    allows for subscription managers who do not have admin roles on the endpoint to set
    the subscription ID.
    """
    client = get_client()

    subscription_id_val: Optional[str]

    if subscription_id == EXPLICIT_NULL:
        subscription_id_val = None
    else:
        subscription_id_val = str(subscription_id)

    doc = {
        "DATA_TYPE": "endpoint_subscription#1.0.0",
        "subscription_id": subscription_id_val,
    }

    res = client.put_endpoint_subscription_id(data=doc)
    record_print_result_message(res)
