import uuid

from gcs_cli.helpers import record_print_result_message, requires_login
from gcs_cli.parsing import globus_command, node_id_arg
from gcs_cli.services.gcs import get_client


@globus_command("delete", short_help="Delete a Node by ID")
@node_id_arg
@requires_login
def node_delete(node_id: uuid.UUID) -> None:
    """
    Delete a Node by ID on the current Endpoint

    This command is not the same as the the node cleanup command available from the
    Node's server itself!

    node cleanup will delete the Node from the Endpoint, but it will also remove it from
    the server where it is run. By contrast, this command will only remove the Node
    from the Endpoint.

    `node delete` is most suitable in cases where the server hosting the node is no
    longer available and `node cleanup` therefore cannot be run. Whenever possible, you
    should prefer to run `node cleanup`.
    """
    client = get_client()
    res = client.delete_node(node_id)
    record_print_result_message(res)
