from typing import Any, Dict, Optional

import click

from gcs_cli.commands.storage_gateway._common import make_create_command
from gcs_cli.parsing import (
    box_options,
    globus_command,
    storage_gateway_create_and_update_params,
)
from gcs_cli.sdk_client.data import BoxStoragePolicies, StorageGatewayDocument
from gcs_cli.services.gcs import BOX_CONNECTOR_ID


@globus_command("box")
@storage_gateway_create_and_update_params(create=True)
@box_options
@make_create_command
def box_create(
    *, box_settings: Optional[Dict[str, Any]], **kwargs: Any
) -> StorageGatewayDocument:
    """Create a new Box Storage Gateway"""
    if not box_settings:
        raise click.UsageError("Missing --box-settings")
    policies = BoxStoragePolicies(
        enterpriseID=box_settings["enterpriseID"],
        boxAppSettings=box_settings["boxAppSettings"],
    )
    return StorageGatewayDocument(
        connector_id=BOX_CONNECTOR_ID, policies=policies, **kwargs
    )
