from typing import TYPE_CHECKING, Any, Iterable, Optional, Tuple

if TYPE_CHECKING:
    import uuid

from gcs_cli.commands.storage_gateway._common import make_update_command
from gcs_cli.parsing import (
    globus_command,
    posix_options,
    storage_gateway_create_and_update_params,
    storage_gateway_id_arg,
)
from gcs_cli.sdk_client.data import POSIXStoragePolicies, StorageGatewayDocument
from gcs_cli.services.gcs import POSIX_CONNECTOR_ID


@globus_command("posix", short_help="Update a POSIX Storage Gateway")
@storage_gateway_id_arg
@storage_gateway_create_and_update_params(create=False)
@posix_options(create=False)
@make_update_command
def posix_update(
    storage_gateway_id: "uuid.UUID",
    *,
    posix_group_allow: Optional[Iterable[str]],
    posix_group_deny: Optional[Iterable[str]],
    **kwargs: Any
) -> Tuple["uuid.UUID", StorageGatewayDocument]:
    """
    Update an existing POSIX Storage Gateway.
    Any options given will replace existing values in the storage gateway's definition.
    Any options not given will retain their existing values.
    """
    policies = None
    if posix_group_allow is not None or posix_group_deny is not None:
        policies = POSIXStoragePolicies(
            groups_allow=posix_group_allow, groups_deny=posix_group_deny
        )

    return (
        storage_gateway_id,
        StorageGatewayDocument(
            connector_id=POSIX_CONNECTOR_ID, policies=policies, **kwargs
        ),
    )
