from globus_sdk.paging import Paginator

from gcs_cli.commands.user_credentials._common import credential_status
from gcs_cli.helpers import default_identity_id_resolver, requires_login
from gcs_cli.parsing import globus_command
from gcs_cli.prettyprint import FORMAT_TEXT_TABLE, formatted_print
from gcs_cli.services.gcs import get_client


@globus_command("list", short_help="List all User Credentials on the Endpoint")
@requires_login
def cred_list() -> None:
    """
    Show all User Credentials on the current Endpoint
    """
    client = get_client()
    res: Paginator = client.paginated.get_user_credential_list()
    formatted_print(
        res,
        text_format=FORMAT_TEXT_TABLE,
        fields=[
            ("ID", "id"),
            ("Local Username", "username"),
            ("Username", default_identity_id_resolver.field),
            ("Status", credential_status),
        ],
    )
