from typing import (
    ClassVar,
    List,
    Type,
    TYPE_CHECKING,
)
import uuid

from .base import (
    Check,
    CheckInvoker,
    MissingPropertyCheck,
    PropertyConstraintCheck,
)

if TYPE_CHECKING:
    from ..models.role import Role  # noqa: F401


class RoleCheck(CheckInvoker["Role"]):
    checkers: ClassVar[List[Type[Check["Role"]]]] = []
    object_name = "role"


@RoleCheck.property_checker(property_name="principal")
class RO01(MissingPropertyCheck["Role"]):
    ...


@RoleCheck.property_checker(property_name="principal")
class RO02(PropertyConstraintCheck["Role"]):
    constraint_name = "principal is UUID"

    def value_is_ok(self, value: "Role") -> bool:
        ok = True
        if value.principal is not None:
            try:
                uuid.UUID(value.principal)
            except ValueError:
                ok = False
        return ok


@RoleCheck.property_checker(property_name="principal_type")
class RO03(MissingPropertyCheck["Role"]):
    ...


@RoleCheck.property_checker(property_name="principal_type")
class RO04(PropertyConstraintCheck["Role"]):
    constraint_name = "principal_type is identity or group"

    def value_is_ok(self, value: "Role") -> bool:
        ok = True
        if value.principal_type is not None:
            if value.principal_type not in ["identity", "group"]:
                ok = False
        return ok


@RoleCheck.property_checker(property_name="role")
class RO05(MissingPropertyCheck["Role"]):
    ...


@RoleCheck.property_checker(property_name="role")
class RO06(PropertyConstraintCheck["Role"]):
    constraint_name = "role is one of the defined role types"

    def value_is_ok(self, value: "Role") -> bool:
        ok = True
        if value.role is not None:
            if value.role not in [
                "administrator",
                "access_manager",
                "activity_manager",
                "activity_monitor",
            ]:
                ok = False
        return ok
