import uuid

import click

from ..models import MigrationPlan
from .decorators import requires_migration_plan


@click.command("read", help="Read sharing state file and display the share's base path")
@click.argument("COLLECTION_ID", required=True, type=click.types.UUIDParameterType())
@requires_migration_plan
def sharing_state_read(collection_id: uuid.UUID, migration_plan: MigrationPlan) -> None:
    gc = migration_plan.get_guest_collection_by_id(collection_id)
    if not gc:
        raise click.ClickException("Unknown collection {collection_id}")
    if gc.disable:
        raise click.ClickException("Collection {collection_id} is disabled")
    gc.load_share_state_file()

    if gc.base_path is None:
        raise click.ClickException(
            f"Unable to load base path for {gc.id} from {gc.share_state_file}"
        )
    click.echo(gc.base_path)
