import uuid

import click

from ..models import MigrationPlan
from .decorators import requires_migration_plan


@click.command(
    "set-base-path",
    help="Set the value of the sharing state file for a guest collection",
)
@click.argument("COLLECTION_ID", required=True, type=click.types.UUIDParameterType())
@click.argument("BASE_PATH", required=True)
@requires_migration_plan
def sharing_state_set_base_path(
    collection_id: uuid.UUID, base_path: str, migration_plan: MigrationPlan
) -> None:
    gc = migration_plan.get_guest_collection_by_id(collection_id)
    if not gc:
        raise click.ClickException("Unknown collection {collection_id}")
    if gc.disable:
        raise click.ClickException("Collection {collection_id} is disabled")
    gc.base_path = base_path
    migration_plan.dump_guest_collections()
