from typing import Optional
from typing_extensions import TypedDict

from ..util import yaml, yaml_object
from ..typing.transfer_documents import (
    RoleDocumentDict,
    PrincipalTypeName,
    RoleTypeName,
)
from ..typing.gcs_documents import RoleDict as GcsRoleDict


class RoleDict(TypedDict):
    id: Optional[str]
    principal: Optional[str]
    principal_type: Optional[PrincipalTypeName]
    role: Optional[RoleTypeName]


@yaml_object(yaml)
class Role:
    def __init__(self, data: Optional[RoleDict] = None) -> None:
        self.id: Optional[str] = None
        self.principal: Optional[str] = None
        self.principal_type: Optional[PrincipalTypeName] = None
        self.role: Optional[RoleTypeName] = None
        self.role_document: Optional[RoleDocumentDict] = None

        if data:
            self.__setstate__(data)

    @property
    def role_document(self) -> Optional[RoleDocumentDict]:
        return self._role_document

    @role_document.setter
    def role_document(self, value: Optional[RoleDocumentDict]) -> None:
        self._role_document = value
        if value:
            if self.id is None:
                self.id = value["id"]
            if self.principal_type is None:
                self.principal_type = value["principal_type"]
            if self.principal is None:
                self.principal = value["principal"]
            if self.role is None:
                self.role = value["role"]

    @staticmethod
    def from_role_document(role_document: RoleDocumentDict) -> "Role":
        role = Role()
        role.role_document = role_document
        return role

    def __eq__(self, other: object) -> bool:
        if not isinstance(other, Role):
            return NotImplemented

        return self is other or self.__getstate__() == other.__getstate__()

    def __getstate__(self) -> RoleDict:
        return {
            "id": self.id,
            "principal": self.principal,
            "principal_type": self.principal_type,
            "role": self.role,
        }

    def __setstate__(self, state: RoleDict) -> None:
        self.id = state["id"]
        self.principal = state["principal"]
        self.principal_type = state["principal_type"]
        self.role = state["role"]

    def to_json(self) -> GcsRoleDict:
        role: GcsRoleDict = {"DATA_TYPE": "role#1.0.0"}

        if self.principal_type and self.principal:
            if self.principal_type == "group":
                principal = f"urn:globus:groups:id:{self.principal}"
            else:
                principal = f"urn:globus:auth:identity:{self.principal}"
            role["principal"] = principal

        if self.role:
            role["role"] = self.role

        return role
