# pragma: no cover
from typing import List, Optional
from typing_extensions import Literal, TypedDict


PrincipalTypeName = Literal["identity", "group", "all_authenticated_users", "anonymous"]

RoleTypeName = Literal[
    "administrator", "access_manager", "activity_manager", "activity_monitor"
]

ACLPermissionName = Literal["r", "rw"]


class AccessDocumentDict(TypedDict):
    DATA_TYPE: Literal["access"]
    id: Optional[int]
    principal_type: PrincipalTypeName
    principal: str
    path: Optional[str]
    permissions: Optional[ACLPermissionName]
    create_time: Optional[str]


NetworkUseName = Literal["normal", "minimal", "aggressive", "custom"]

EndpointServerSchemeName = Literal["ftp", "gsiftp"]


class AccessListDocumentDict(TypedDict):
    DATA_TYPE: Literal["access_list"]
    length: int
    endpoint: str
    DATA: List[AccessDocumentDict]


class RoleDocumentDict(TypedDict):
    DATA_TYPE: Literal["role"]
    id: str
    principal_type: PrincipalTypeName
    principal: str
    role: RoleTypeName


class RoleListDocumentDict(TypedDict):
    DATA_TYPE: Literal["role_list"]
    DATA: List[RoleDocumentDict]


class EndpointDocumentDict(TypedDict):
    DATA_TYPE: Literal["endpoint"]
    id: str
    display_name: Optional[str]
    organization: Optional[str]
    department: Optional[str]
    keywords: Optional[str]
    name: str
    canonical_name: str
    username: str
    owner_id: str
    owner_string: str
    description: Optional[str]
    contact_email: Optional[str]
    contact_info: Optional[str]
    info_link: Optional[str]
    user_message: Optional[str]
    user_message_link: Optional[str]
    public: bool
    subscription_id: Optional[str]
    french_english_bilingual: bool
    default_directory: Optional[str]
    force_encryption: bool
    disable_verify: bool
    force_verify: bool
    mfa_required: bool
    expire_time: Optional[str]
    expires_in: int
    activated: bool
    myproxy_server: Optional[str]
    myproxy_dn: Optional[str]
    oauth_server: Optional[str]
    is_globus_connect: bool
    gcs_version: Optional[str]
    globus_connect_setup_key: Optional[str]
    host_endpoint: Optional[str]
    host_endpoint_id: Optional[str]
    host_endpoint_display_name: Optional[str]
    host_path: Optional[str]
    s3_url: Optional[str]
    s3_owner_activated: bool
    acl_available: bool
    acl_editable: bool
    in_use: bool
    my_effective_roles: List[str]
    gcp_connected: Optional[bool]
    gcp_paused: Optional[bool]
    network_use: Optional[NetworkUseName]
    location: Optional[str]
    max_concurrency: Optional[int]
    preferred_concurrency: Optional[int]
    max_parallelism: Optional[int]
    preferred_parallelism: Optional[int]
    local_user_info_available: Optional[bool]
    https_server: Optional[str]
    high_assurance: bool
    authentication_timeout_mins: Optional[int]
    authentication_assurance_timeout: Optional[int]
    DATA: List["EndpointServerDocumentDict"]
    non_functional: bool
    shareable: bool


class EndpointServerDocumentDict(TypedDict):
    DATA_TYPE: Literal["server"]
    id: int
    hostname: str
    port: int
    scheme: EndpointServerSchemeName
    subject: Optional[str]
    incoming_data_port_start: int
    incoming_data_port_end: int
    outgoing_data_port_start: int
    outgoing_data_port_end: int
    uri: str
    is_connected: bool
    is_paused: bool


class EffectivePermissionsDict(TypedDict):
    DATA_TYPE: Literal["effective_permission"]
    encoded_path: str
    permissions: ACLPermissionName


class AnonymousEffectivePermissionsDict(TypedDict):
    DATA_TYPE: Literal["effective_permissions"]
    DATA: List[EffectivePermissionsDict]
    user_cert_chain: str
    sharing_uuid: str


class SharedEndpointListEntry(TypedDict):
    id: str


class SharedEndpointListDict(TypedDict):
    next_token: Optional[str]
    shared_endpoints: List[SharedEndpointListEntry]
