from typing import List

import marshmallow as ma

from ..api.collection import CollectionSchema
from ..api.data_type import DataTypeSchema
from ..api.storage_gateway import StorageGatewaySchema
from ..api.user_credential import UserCredentialSchema


S3_CONNECTOR_ID = "7643e831-5f6c-4b47-a07f-8ee90f401d23"


class S3StoragePoliciesSchema(ma.Schema):
    _DATA_TYPE = "s3_storage_policies#1.0.0"
    DATA_TYPE = ma.fields.Str(
        required=True,
        allow_none=False,
        default=_DATA_TYPE,
        validate=[ma.validate.Equal(_DATA_TYPE)],
    )
    s3_endpoint = ma.fields.Url()
    s3_buckets = ma.fields.List(ma.fields.String)
    s3_user_credential_required = ma.fields.Boolean()
    private = []  # type: List[str]
    connector_id = S3_CONNECTOR_ID


class S3CollectionPoliciesSchema(ma.Schema):
    _DATA_TYPE = "s3_collection_policies#1.0.0"
    DATA_TYPE = ma.fields.Str(
        required=True,
        allow_none=False,
        default=_DATA_TYPE,
        validate=[ma.validate.Equal(_DATA_TYPE)],
    )
    private = []  # type: List[str]
    connector_id = S3_CONNECTOR_ID


class S3UserCredentialPoliciesSchema(ma.Schema):
    _DATA_TYPE = "s3_user_credential_policies#1.0.0"
    DATA_TYPE = ma.fields.Str(
        required=True,
        allow_none=False,
        default=_DATA_TYPE,
        validate=[ma.validate.Equal(_DATA_TYPE)],
    )
    s3_key_id = ma.fields.String(allow_none=True)
    s3_secret_key = ma.fields.String(allow_none=True)

    private = ["s3_secret_key"]
    connector_id = S3_CONNECTOR_ID


s3_storage_policies_schema = S3StoragePoliciesSchema(
    load_only=S3StoragePoliciesSchema.private
)
s3_storage_policies_private_schema = S3StoragePoliciesSchema()

s3_user_credential_policies_private_schema = S3UserCredentialPoliciesSchema()
s3_user_credential_policies_schema = S3UserCredentialPoliciesSchema(
    load_only=S3UserCredentialPoliciesSchema.private
)
s3_collection_policies_schema = S3CollectionPoliciesSchema(
    load_only=S3CollectionPoliciesSchema.private
)
s3_collection_policies_private_schema = S3CollectionPoliciesSchema()

DataTypeSchema.register_schema(s3_storage_policies_schema)
DataTypeSchema.register_schema(s3_user_credential_policies_schema)
DataTypeSchema.register_schema(s3_collection_policies_schema)

StorageGatewaySchema.register_schema(
    public=s3_storage_policies_schema, private=s3_storage_policies_private_schema
)

UserCredentialSchema.register_schema(
    public=s3_user_credential_policies_schema,
    private=s3_user_credential_policies_private_schema,
)

CollectionSchema.register_schema(
    public=s3_collection_policies_schema, private=s3_collection_policies_private_schema
)
