from oic.oic.message import OpenIDSchema
from oic.oauth2.message import SINGLE_OPTIONAL_STRING
from oic.oic.provider import Provider
from oic.utils.authz import AuthzHandling
from oic.utils.authn.authn_context import AuthnBroker, PASSWORD
from oic.utils.authn.client import verify_client
from oic.utils.keyio import keyjar_init

from globus_oidc.session_db import make_session_db
from globus_oidc.authentication import pam_authentication
from globus_oidc.config import settings


class CustomOpenIDSchema(OpenIDSchema):
    """Custom Schema object that adds support for the 'organization'
    parameter.
    """
    c_param = dict(OpenIDSchema.c_param, organization=SINGLE_OPTIONAL_STRING)


def make_provider():
    config = settings()['config']

    domain = config['auth_client']['domain']
    base_url = "https://{}".format(domain)
    authorization = AuthzHandling()
    authentication = AuthnBroker()
    authentication.add(PASSWORD, pam_authentication(base_url), 10, base_url)
    userinfo = None # Not supported
    # Copying the client db because it's currently immutable.
    client_db = dict(config['clients'])
    for k in client_db:
        client_db[k] = dict(client_db[k])

    password = config['oidc_server']['shared_key']
    authorization_code_lifetime = 60
    access_token_lifetime = 60 * 15
    session_db = make_session_db(
        password,
        authorization_code_lifetime,
        access_token_lifetime,
        domain
    )

    provider = Provider(
        base_url,
        session_db,
        client_db,
        authentication,
        userinfo,
        authorization,
        verify_client,
        None,
        schema=CustomOpenIDSchema,
    )

    jwks = keyjar_init(provider, config['KEY_CONFIG'])

    return provider, jwks, base_url
